==> openshift-serverless-logic-1.26.0-src.zip: openshift-serverless-logic-1.26.0.GA-src/org.kie.kogito-kogito-runtimes-1.30.0.Final/kogito-serverless-workflow/kogito-serverless-workflow-builder/src/main/java/org/kie/kogito/serverless/workflow/parser/FunctionTypeHandlerFactory.java <==
/*
 * Copyright 2022 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.kie.kogito.serverless.workflow.parser;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.ServiceLoader;

import io.serverlessworkflow.api.functions.FunctionDefinition;

public class FunctionTypeHandlerFactory {

    public static FunctionTypeHandlerFactory instance() {
        return INSTANCE;
    }

    private static final FunctionTypeHandlerFactory INSTANCE = new FunctionTypeHandlerFactory();
    private static final String CUSTOM_TYPE_SEPARATOR = ":";

    private final Map<String, FunctionTypeHandler> typeMap = new HashMap<>();
    private final Map<String, FunctionTypeHandler> customMap = new HashMap<>();

    public Optional<FunctionTypeHandler> getTypeHandler(FunctionDefinition functionDef) {
        final boolean isCustom = functionDef.getType() == FunctionDefinition.Type.CUSTOM;
        return Optional.ofNullable(getMap(isCustom).get(isCustom ? getTypeFromOperation(functionDef) : functionDef.getType().toString()));
    }

    private static String getTypeFromOperation(FunctionDefinition functionDef) {
        String operation = functionDef.getOperation();
        int indexOf = operation.indexOf(CUSTOM_TYPE_SEPARATOR);
        return indexOf == -1 ? operation : operation.substring(0, indexOf);
    }

    public static String trimCustomOperation(FunctionDefinition functionDef) {
        String operation = functionDef.getOperation();
        int indexOf = operation.indexOf(CUSTOM_TYPE_SEPARATOR);
        return indexOf == -1 ? operation : operation.substring(indexOf + 1);
    }

    private Map<String, FunctionTypeHandler> getMap(boolean isCustom) {
        return isCustom ? customMap : typeMap;
    }

    private FunctionTypeHandlerFactory() {
        ServiceLoader.load(FunctionTypeHandler.class).forEach(handler -> getMap(handler.isCustom()).put(handler.type(), handler));
    }
}

==> openshift-serverless-logic-1.28.0-src.zip: openshift-serverless-logic-1.28.0.GA-src/org.kie.kogito-kogito-runtimes-1.35.0.Final/kogito-serverless-workflow/kogito-serverless-workflow-builder/src/main/java/org/kie/kogito/serverless/workflow/parser/FunctionTypeHandlerFactory.java <==
/*
 * Copyright 2022 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.kie.kogito.serverless.workflow.parser;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.ServiceLoader;
import java.util.function.BiFunction;

import io.serverlessworkflow.api.functions.FunctionDefinition;

public class FunctionTypeHandlerFactory {

    public static FunctionTypeHandlerFactory instance() {
        return INSTANCE;
    }

    private static final FunctionTypeHandlerFactory INSTANCE = new FunctionTypeHandlerFactory();
    private static final String CUSTOM_TYPE_SEPARATOR = ":";

    // we need two maps to avoid clashing
    private final Map<String, FunctionTypeHandler> typeMap = new HashMap<>();
    private final Map<String, FunctionTypeHandler> customMap = new HashMap<>();

    public Optional<FunctionTypeHandler> getTypeHandler(FunctionDefinition functionDef) {
        final boolean isCustom = functionDef.getType() == FunctionDefinition.Type.CUSTOM;
        Map<String, FunctionTypeHandler> map;
        String key;
        if (isCustom) {
            map = customMap;
            key = getTypeFromOperation(functionDef);
        } else {
            map = typeMap;
            key = functionDef.getType().toString();
        }
        return Optional.ofNullable(map.get(key));
    }

    private static String getTypeFromOperation(FunctionDefinition functionDef) {
        return trimString(functionDef, FunctionTypeHandlerFactory::trimStart);
    }

    private static String trimStart(String operation, int indexOf) {
        return operation.substring(0, indexOf);
    }

    private static String trimEnd(String operation, int indexOf) {
        return operation.substring(indexOf + 1);
    }

    private static String trimString(FunctionDefinition functionDef, BiFunction<String, Integer, String> function) {
        String operation = functionDef.getOperation();
        int indexOf = operation.indexOf(CUSTOM_TYPE_SEPARATOR);
        return indexOf == -1 ? operation : function.apply(operation, indexOf);
    }

    public static String trimCustomOperation(FunctionDefinition functionDef) {
        return trimString(functionDef, FunctionTypeHandlerFactory::trimEnd);
    }

    private FunctionTypeHandlerFactory() {
        ServiceLoader.load(FunctionTypeHandler.class).forEach(handler -> (handler.isCustom() ? customMap : typeMap).put(handler.type(), handler));
    }
}
